import unittest
import numpy as np

from src.spatial_light_controller import SpatialLightController
from src.director import Director
from src.output_devices.dimmer_interface import DimmerInterface


class TestSpatialLightingController(unittest.TestCase):
    """
    The spatial light controller is responsible for running each module that generates output,
    then sends the outputs generated by the current frame cycle into the sequencer
    """

    def setup_dependencies(self):
        self.director = Director()
        # TODO add these outputs to a fixtures file
        # single value change - 1 channel, 1 item in sequence
        self.light_controller = SpatialLightController(
            director=self.director,
            send_channel_message=lambda x: x,
        )

        self.spatial_categories = [
            "left",
            "right",
            "top",
            "bottom",
            "back",
            "front",
            "middle",
        ]

    def teardown_dependencies(self):
        self.director = None
        self.sequencer = None
        self.one_channel_output = []
        self.two_channel_output = []
        self.three_channel_output = []


if __name__ == "__main__":
    unittest.main()
